import 'package:daggerito/daggerito.dart';

class Container {
  static Component _appScopedComponentInstance = Component();
  final Component _appScopedComponent;
  Component _sessionScopedComponent;
   Component get appComponent => _appScopedComponent;
   Component get sessionComponent => _sessionScopedComponent;
   void set sessionComponent(Component component) => _sessionScopedComponent = component;
  Component get mainComponent =>  _mainComponent ??= SubComponent(_appScopedComponent);

  void set mainComponent(Component component) => _mainComponent = SubComponent(_appScopedComponent, container: component.container);

  DependencyContainer get appScopedDependencyContainer =>
      _appScopedComponent.container;

  Component _mainComponent;

  DependencyContainer get mainDependencyContainer =>
     mainComponent.container;

  Container({Component component})
      : _appScopedComponent = component != null
            ? SubComponent(_appScopedComponentInstance, container: component.container)
            : _appScopedComponentInstance;

  T checkAndSetSessionComponent<T extends Component>(String tag){
    if (has<T>(tag, DependencyScope.app)){
      sessionComponent = resolve<T>(tag, DependencyScope.app);
      return sessionComponent;
    }
    return null;
  }

  bool has<T>([String tag, DependencyScope scope = DependencyScope.view]) {
    switch (scope) {
      case DependencyScope.app:
        return appScopedDependencyContainer.has<T>(tag);
      case DependencyScope.session:
        return false;
      case DependencyScope.view:
        return mainDependencyContainer.has<T>(tag);
    }
  }

  void register<S, T extends S>(Factory<S> factory,
      {String tag, Type as, DependencyScope scope = DependencyScope.view}) {
    switch (scope) {
      case DependencyScope.app:
        appScopedDependencyContainer.setDefinition(
            tag, Definition<S>.factory(factory), as);
        break;
      case DependencyScope.session:
        throw "Sesijska se ne moze ovako registrovati";
      case DependencyScope.view:
        mainDependencyContainer.setDefinition(
            tag, Definition<S>.factory(factory), as);
    }
  }

  void registerSingleton<S, T extends S>(Factory<S> factory,
      {String tag, Type as, DependencyScope scope = DependencyScope.view}) {
    switch (scope) {
      case DependencyScope.app:
        appScopedDependencyContainer.setDefinition(
            tag, Definition<S>.singleton(factory), as);
        break;
      case DependencyScope.session:
        throw "Sesijska se ne moze ovako registrovati";
      case DependencyScope.view:
        mainDependencyContainer.setDefinition(
            tag, Definition<S>.singleton(factory), as);
    }
  }

  T call<T>([String tag, DependencyScope scope = DependencyScope.view]) =>
      resolve<T>(tag, scope);

  T resolve<T>([String tag, DependencyScope scope = DependencyScope.view]) {
    switch (scope) {
      case DependencyScope.app:
        return appScopedDependencyContainer.resolve<T>(tag);
      case DependencyScope.session:
        throw "Sesijsku posevno definirati";
      case DependencyScope.view:
        return mainDependencyContainer.resolve<T>(tag);
    }
  }
}

enum DependencyScope {
  app,
  session,
  view,
}
