
import 'package:routex_core/src/web/http_server_request.dart';
import 'package:routex_core/src/web/http_server_response.dart';

import 'container/container.dart';
import 'route.dart';

/// Represents the context for the handling of a request in Routex.
/// A new instance is created for each request that is received in the [Router.handle] of the router.
abstract class RoutingContext<T extends Container> {
  HttpServerRequest request();

  HttpServerResponse response();

  void next();

  String normalisedPath();

  bool failed();

  void fail(Object object);

  RoutingContext put(String key, dynamic obj);

  T get<T>(String key);

  Map<String, dynamic> data();

  T getParam<T>(String key);

  Map<String, dynamic> params();

  Object failure;

  int get statusCode;

  void reroute(String path);

  String mountPoint();

  Route currentRoute();

  Map<String, dynamic> getBodyAsJson();

  User user();

  void setUser(User user);

  T container;

  T call<T>([String tag, DependencyScope scope = DependencyScope.view]) =>
      container.resolve<T>(tag, scope);
}

abstract class User<T> {
  String get id;

  T get payload;
}
